/*
 * showkeys: Print out all the keys, and the number of values for each key,
 * in the ctfnode.db file.
 * Copyright (c) Warren Toomey, under the GPL license.
 * 
 * $Revision: 1.12 $
 */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include "dbfiles.h"
#include "tokens.h"

int 
main(int argc, char *argv[])
{
  uint8_t *key, *oldkey;
  Ctfhashnode *node;
  int showvalues = 0;
  int numvals;
  int i;

  /* Set the value flag if an argument */
  if (argc == 2 && !strcmp(argv[1], "-v")) showvalues = 1;

  /* Get first key */
  if (get_firstkey(&key, &node, &numvals)==0) {
    printf("Cannot get first key\n"); exit(1);
  }

  /* Loop while we have a key */
  while (1) {

    /* Print out # values and key */
    printf("%4d\t", numvals);
    for (i=0; i< TUPLE_SIZE; i++) printf("%02x", key[i]);
    printf(":%02hhx%02hhx\n", key[TUPLE_SIZE], key[TUPLE_SIZE+1]);

    /* Print out each value if required */
    if (showvalues) {
      printf("\t");
      for (i=0; i< TUPLE_SIZE; i++) printf("%s", tok2str(key[i]));
      printf("\n");

      for (i = 0; i < numvals; i++) {
	printf(
	"\tValue %d:\toffset %04x name %04x file %02d lines %03d to %03d\n", i,
	       node[i].offset, node[i].name_offset, node[i].fileid,
		node[i].line_number, node[i].last_line_number);
      }
      printf("\n");
    }
#ifdef GDBM
    free(node);
#endif
    oldkey= key;
    if (get_nextkey(oldkey, &key, &node, &numvals)==0)
      break;
#ifdef GDBM
    free(oldkey);
#endif
  }
  return (0);
}
